#!/usr/bin/perl
#++
#   $Id$
# 
#   File:		mid2delay.pl
#   Date:		Fri Mar 23 22:28:51 2007
#   Author:		Alexander Zangerl (az)
# 
#   Abstract:
#	make freq+delay pic code from midi file
#
#   Modifications:
#	$Log$
#--
use strict;

my ($midfile,$divider)=@ARGV;
die "usage: $0 midfile divider\n" if (!-r $midfile || !$divider);
open(F,"mftext $midfile|");

my $factor=255/480/$divider;

my ($then,$now,@notes,@durations);
while (<F>)
{
    my ($midi);

    chomp;
    # Time=1200  Note on, chan=1 pitch=64 vol=64
    if (/Time=(\d+)\s+Note on,/)
    {
	$then=$1;
	if (defined($now) && $now != $then)
	{
	    # silence
	    push @notes,0;
	    push @durations,int($factor*($then-$now));
	    next;
	}
    }
    /Time=(\d+)\s+Note off,.*pitch=(\d+)/ && (($now,$midi)=($1,$2)) || next;
#    print "line $_ midi $midi\n";

    my $freq=440*exp(log(2)*($midi-69)/12);
    my $delay=int(0.5+1e6/(26*$freq));

    my $delta=int($factor*($now-$then));

    if ($delta>255)
    {
	$delta=255;
    }

    push @notes,$delay;
    push @durations,$delta;
}

push @notes,0;
push @durations,0;


printf "TSIZE equ .".scalar(@notes)."\n";

print "notes\taddwf PCL,F\n";
print map { "\tretlw .$_\n"; } (@notes);

print "\n\nduration\taddwf PCL,F\n";
print map { "\tretlw .$_\n"; } (@durations);

